@@ -298,48 +298,36 @@ from Page unless overwritten) are:
 ###############################################################################
 package XAO::DO::Web::Page;
 use strict;
-use Error qw(:try);
 use XAO::Utils;
+use XAO::Cache;
 use XAO::Templates;
+use XAO::Objects;
 use XAO::Projects qw(:all);
 use XAO::PageSupport;
+use Error qw(:try);
+
+use base XAO::Objects->load(objname => 'Atom');
 
-##
-# Package version
-#
 use vars qw($VERSION);
-($VERSION)=(q$Id: Page.pm,v 1.7 2002/01/04 03:27:25 am Exp $ =~ /(\d+\.\d+)/);
+($VERSION)=(q$Id: Page.pm,v 1.13 2002/02/20 01:03:29 am Exp $ =~ /(\d+\.\d+)/);
 
 ##
-# Methods prototypes
+# Prototypes
 #
-sub new ($%);
-sub display ($%);
-sub expand ($%);
-sub parse ($%);
-sub object ($%);
-sub parse_args ($);
-sub editable ();
-sub check_db ($);
-sub textout ($%);
-sub finaltextout ($%);
-sub dbh ($);
-sub odb ($);
-sub siteconfig ($);
+sub cache ($%);
 sub cgi ($);
-
-###############################################################################
-# Creating new instance of Page.
-#
-sub new ($%) {
-    my $proto=shift;
-    my $class=ref($proto) || $proto;
-    my $self=get_args(\@_) || {};
-    bless $self,$class;
-    $self->{sitename}=get_current_project_name();
-    $self->{siteconfig}=get_current_project();
-    $self;
-}
+sub check_db ($);
+sub dbh ($);
+sub display ($%);
+sub editable ();
+sub expand ($%);
+sub finaltextout ($%);
+sub object ($%);
+sub odb ($);
+sub parse ($%);
+sub parse_args ($);
+sub siteconfig ($);
+sub textout ($%);
 
 ###############################################################################
 
@@ -431,7 +419,12 @@ sub display ($%) {
         }
         catch XAO::E::Objects with {
             my $e=shift;
-            eprint "Object loading error while processing path='$args->{path}'";
+            if($args->{path}) {
+                eprint "Object loading error while processing path='$args->{path}'";
+            }
+            else {
+                eprint "Object loading error";
+            }
             $e->throw;
         };
 
@@ -677,9 +670,9 @@ Example:
 sub dbh ($) {
     my $self=shift;
     return $self->{dbh} if $self->{dbh};
-    $self->{dbh}=$self->{siteconfig}->dbh;
+    $self->{dbh}=$self->siteconfig->dbh;
     return $self->{dbh} if $self->{dbh};
-    throw XAO::Errors::Page ref($self)." requires database connection";
+    throw $self "dbh - no database connection";
 }
 
 ###############################################################################
@@ -704,11 +697,10 @@ sub odb ($) {
     my $self=shift;
     return $self->{odb} if $self->{odb};
 
-    $self->{odb}=$self->{siteconfig}->odb;
+    $self->{odb}=$self->siteconfig->odb;
     return $self->{odb} if $self->{odb};
 
-    throw XAO::Errors::Page
-          ref($self)."odb - requires object database connection";
+    throw $self "odb - requires object database connection";
 }
 
 ###############################################################################
@@ -752,7 +744,9 @@ data between objects. See L<XAO::Projects> for more details.
 
 sub siteconfig ($) {
     my $self=shift;
-    $self->{siteconfig};
+    return $self->{siteconfig} if $self->{siteconfig};
+    $self->{siteconfig}=$self->{sitename} ? get_project($self->{sitename})
+                                          : get_current_project();
 }
 
 ###############################################################################
@@ -784,7 +778,7 @@ sub base_url ($;%) {
     my $args=get_args(\@_);
     my $url;
     my $secure=$args->{secure};
-    $secure=$self->cgi->https() ? 1 : 0 unless defined $secure;
+    $secure=$self->is_secure unless defined $secure;
     if($secure) {
         $url=$self->siteconfig->get('base_url_secure');
         if(!$url) {
@@ -799,6 +793,20 @@ sub base_url ($;%) {
 
 ###############################################################################
 
+=item is_secure ()
+
+Returns 1 if the current the current connection is a secure one or 0
+otherwise.
+
+=cut
+
+sub is_secure ($) {
+    my $self=shift;
+    return $self->cgi->https() ? 1 : 0;
+}
+
+###############################################################################
+
 =item pageurl (%)
 
 Returns full URL of current page without parameters. Accepts the same
@@ -840,6 +848,7 @@ Example -- setting new path and leaving all other arguments untouched:
 
 sub merge_args ($%) {
     my $self=shift;
+    eprint ref($self)."::merge_args - obsolete, use XAO::Utils::merge_refs";
     my $args=get_args(\@_);
     my $oldargs=$args->{oldargs};
     my $newargs=$args->{newargs};
@@ -866,137 +875,147 @@ sub merge_args ($%) {
 #  , { text => text }
 #  ]
 #
-my %parsed_cache;
-sub parse ($%)
-{ my ($self,%args)=@_;
-  my $classname=ref $self || $self;
-  if(! keys %args)
-   { eprint "$classname : No arguments given";
-     return undef;
-   }
-
-  ##
-  # Getting template text
-  #
-  my $template;
-  if(defined($args{template}))
-   { $template=$args{template};
-   }
-  else
-   { my $path=$args{path};
-     if(! $path)
-      { throw XAO::Errors::Page ref($self)."::parse - No path given to Page object";
+### my %parsed_cache;
+sub parse ($%) {
+    my ($self,%args)=@_;
+    my $classname=ref $self || $self;
+    if(! keys %args) {
+        eprint "$classname : No arguments given";
         return undef;
-      }
-     return $parsed_cache{$path} if exists($parsed_cache{$path});
-     $template=XAO::Templates::get(path => $path);
-     defined($template) || throw XAO::Errors::Page
-                                 ref($self)."::parse - no template found (path=$path)";
-   }
+    }
 
-  ##
-  # Checking if we do not need to parse that template.
-  #
-  if($self->{args}->{unparsed})
-   { return [ { text => $template } ];
-   }
+    ##
+    # Getting template text
+    #
+    my $template;
+    if(defined($args{template})) {
+        $template=$args{template};
+    }
+    else {
+        my $path=$args{path} ||
+            throw $self "parse - No path given to a Page object";
 
-  ##
-  # Parsing
-  #
-  my @page;
-  $template=~s/<!--(?!\/\/).*?-->//sg;
-  my @parts=split('(<%|%>|"|{|})',$template);
-  my $in_object=0;
-  my $in_quotes=0;
-  my $in_brackets=0;
-  my $objtext;
-  for(my $pnum=0; $pnum!=@parts; $pnum++)
-   { my $part=$parts[$pnum];
-     if($in_object)
-      { if($in_brackets)
-         { $objtext.=$part;
-           if($part eq '{')
-            { $in_brackets++;
-            }
-           elsif($part eq '}')
-            { $in_brackets--;
-            }
-         }
-        elsif($part eq '"')
-         { $objtext.='"';
-           if($in_quotes)
-            { $in_quotes=0;
-            }
-           else
-            { $in_quotes=1;
-            }
-         }
-        elsif($in_quotes)
-         { $objtext.=$part;
-         }
-        elsif($part eq '{')
-         { $in_brackets++;
-           $objtext.='{';
-         }
-        elsif($part eq '<%')
-         { $in_object++;
-           $objtext.=$part;
-         }
-        elsif($part eq '%>')
-         { $in_object--;
-           if(!$in_object)
-            { push(@page,{ objtext => $objtext });
-            }
-           else
-            { $objtext.=$part;
-            }
-         }
-        else
-         { $objtext.=$part;
-         }
-      }
-     else
-      { if($part eq '<%')
-         { $in_object++;
-           $objtext='';
-         }
-        else
-         { push(@page,{ text => $part });
-         }
-      }
-   }
-  throw XAO::Errors::Page ref($self).'::display - not closed object in template' if $in_object;
-  foreach my $item (@page)
-   { next unless defined($item->{objtext});
-     if($item->{objtext} !~ /^\s*(\w[\w\.:]*)(\/(\w+))?\s*(.*)$/s)
-      { $item->{text}='<%';	# <%%> is just a funny way to embed <%
-        delete $item->{objtext};
-        next;
-      }
-     $item->{objname}=$1;
-     $item->{flag}=$3 ? $item->{flag}=lc(substr($3,0,1)) : 't';
-     $item->{args}=parse_args($4);
-   }
+        if($self->debug_check('show-path')) {
+            dprint $self->{objname}."::parse - path='$path'";
+        }
 
-  ##
-  # Document structure
-  #
-  ## for(my $i=0; $i!=@page; $i++)
-  ##  { dprint "$i) ",join(",",%{$page[$i]}),"\n";
-  ##    if($page[$i]->{args})
-  ##     { my $args=$page[$i]->{args};
-  ##       foreach my $a (sort keys %{$args})
-  ##        { dprint "   $a => $args->{$a}\n";
-  ##        }
-  ##     }
-  ##  }
+        #XXX# return $parsed_cache{$path} if exists($parsed_cache{$path});
 
-  ##
-  # Storing into cache and returning
-  #
-  return ($parsed_cache{$args{path}}=\@page) unless exists($args{template});
-  \@page;
+        $template=XAO::Templates::get(path => $path);
+        defined($template) ||
+            throw $self "parse - no template found (path=$path)";
+    }
+
+    ##
+    # Checking if we do not need to parse that template.
+    #
+    if($self->{args}->{unparsed}) {
+        return [ { text => $template } ];
+    }
+
+    ##
+    # Parsing
+    #
+    my @page;
+    $template=~s/<!--(?!\/\/).*?-->//sg;
+    my @parts=split('(<%|%>|"|{|})',$template);
+    my $in_object=0;
+    my $in_quotes=0;
+    my $in_brackets=0;
+    my $objtext;
+    for(my $pnum=0; $pnum!=@parts; $pnum++) {
+        my $part=$parts[$pnum];
+        if($in_object) {
+            if($in_brackets) {
+                $objtext.=$part;
+                if($part eq '{') {
+                    $in_brackets++;
+                }
+                elsif($part eq '}') {
+                    $in_brackets--;
+                }
+            }
+            elsif($part eq '"') {
+                $objtext.='"';
+                if($in_quotes) {
+                    $in_quotes=0;
+                }
+                else {
+                    $in_quotes=1;
+                }
+            }
+            elsif($in_quotes) {
+                $objtext.=$part;
+            }
+            elsif($part eq '{') {
+                $in_brackets++;
+                $objtext.='{';
+            }
+            elsif($part eq '<%') {
+                $in_object++;
+                $objtext.=$part;
+            }
+            elsif($part eq '%>') {
+                $in_object--;
+                if(!$in_object) {
+                    push(@page,{ objtext => $objtext });
+                }
+                else {
+                    $objtext.=$part;
+                }
+            }
+            else {
+                $objtext.=$part;
+            }
+        }
+        else {
+            if($part eq '<%') {
+                $in_object++;
+                $objtext='';
+            }
+            else {
+                push(@page,{ text => $part });
+            }
+        }
+    }
+
+    $in_object && throw $self 'display - not closed object in template';
+
+    foreach my $item (@page) {
+        next unless defined($item->{objtext});
+        if($item->{objtext} !~ /^\s*(\w[\w\.:]*)(\/(\w+))?\s*(.*)$/s) {
+            $item->{text}='<%';	# <%%> is just a funny way to embed <%
+            delete $item->{objtext};
+            next;
+        }
+        $item->{objname}=$1;
+        $item->{flag}=$3 ? $item->{flag}=lc(substr($3,0,1)) : 't';
+        $item->{args}=parse_args($4);
+    }
+
+    ##
+    # Document structure
+    #
+    ## for(my $i=0; $i!=@page; $i++)
+    ##  { dprint "$i) ",join(",",%{$page[$i]}),"\n";
+    ##    if($page[$i]->{args})
+    ##     { my $args=$page[$i]->{args};
+    ##       foreach my $a (sort keys %{$args})
+    ##        { dprint "   $a => $args->{$a}\n";
+    ##        }
+    ##     }
+    ##  }
+
+    ## XXX - Cache have to be much more elaborate, the same template can
+    ## be parsed into different things and we can't cache it without
+    ## content checks. Need to check if hashing by complete content is
+    ## faster then just re-parsing.
+    ## ##
+    ## # Storing into the cache if possible
+    ## #
+    ## $parsed_cache{$args{path}}=\@page unless defined($args{template});
+
+    \@page;
 }
 
 ##
@@ -1128,50 +1147,85 @@ sub parse_args ($)
 
 ##
 # This is overriden in all editable objects. Default is "not editable".
+# XXX - unused.
 #
-sub editable ()
-{ return 0;
+sub editable () {
+    eprint "editable - should not be used any more";
+    return 0;
 }
 
-##
-# Checks that we have database connection. Suitable in derived objects.
-#
-sub check_db ($)
-{ my $self=shift;
-  eprint ref($self)."::check_db method is obsolete, use dbh() to get dbh";
-  $self->dbh;
+###############################################################################
+
+=item cache (%)
+
+Creates or retrieves a cache for use in various Page based
+objects. Arguments are directly passed to XAO::Cache's new() method (see
+L<XAO::Cache>) except for 'name' argument which is used to identify the
+requested cache.
+
+If a cache with that name was already initialized before it is not
+re-created, but previously created version is returned instead.
+
+Example:
+
+ my $cache=$self->cache(
+     name        => 'fubar',
+     retrieve    => \&real_retrieve,
+     coords      => ['foo','bar'],
+     expire      => 60
+ );
+
+=cut
+
+sub cache ($%) {
+    my $self=shift;
+    my $args=get_args(\@_);
+
+    my $name=$args->{name} ||
+        throw $self "cache - no 'name' argument";
+
+    my $cache_list=$self->siteconfig->get('cache_list');
+    if(! $cache_list) {
+        $cache_list={};
+        $self->siteconfig->put(cache_list => $cache_list);
+    }
+
+    my $cache=$cache_list->{$name};
+    if(! $cache) {
+        $cache=XAO::Cache->new($args);
+        $cache_list->{$name}=$cache;
+    }
+
+    return $cache;
 }
 
-##
-# Helps to write code like:
-#  $sth || throw $self "show_path - SQL error";
-#
-sub throw ($@)
-{ my $self=shift;
-  throw XAO::Errors::Page ref($self) . '::' . join('',@_);
+sub debug_check ($$) {
+    my $self=shift;
+    my $type=shift;
+    return $self->clipboard->get("debug/Web/Page/$type");
 }
 
-##
-# Error to be thrown from displayable objects.
-#
-package XAO::Errors::Page;
-use Error;
-use vars qw(@ISA);
-@ISA=qw(Error::Simple);
+sub debug_set ($%) {
+    my $self=shift;
+    my $args=get_args(\@_);
+    foreach my $type (keys %$args) {
+        $self->clipboard->put("debug/Web/Page/$type",$args->{$type} ? 1 : 0);
+    }
+}
 
-##
-# That's it
-#
+###############################################################################
 1;
 __END__
 
+=back
+
 =head1 EXPORTS
 
 Nothing.
 
 =head1 AUTHOR
 
-Copyright (c) 2000-2001 Xao, Inc.
+Copyright (c) 2000-2002 XAO, Inc.
 
 Andrew Maltsev <am@xao.com>.
 
